import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import {
  csv,
  scaleLinear,
  min,
  max,
  axisLeft,
  axisBottom,
  format,
  extent,
} from 'd3';
import { selectColor } from './selectColor';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';

const height = 500;
const width = 960;
const padding = 20;
const centerX = width / 2;
const centerY = height / 2;
const margin = {
  top: 70,
  bottom: 70,
  left: 100,
  right: 70,
};
const tickOffset = 5;
const xAxisLabelOffset = 50;
const yAxisLabelOffset = -150;
const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;
const xValue = (d) => d.sepal_length;
const yValue = (d) => d.sepal_width;

const xAxisLabel = 'Sepal Length';
const yAxisLabel = 'Sepal Width';

const siFormat = () => format('.2s');
const xAxisTickFormat = (tickValue) =>
  siFormat('.2s')(tickValue).replace('G', 'B');

const App = () => {
  const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={tickOffset}
        />
        <AxisLeft
          yScale={yScale}
          innerWidth={innerWidth}
          tickOffset={tickOffset}
        />
        <text
          x={innerWidth / 2}
          style={{ textAnchor: 'middle' }}
          y={innerHeight + xAxisLabelOffset}
          className="axis-label"
        >
          {xAxisLabel}
        </text>
        <text
          x={yAxisLabelOffset}
          style={{ textAnchor: 'middle' }}
          y={-innerHeight / 2 + 120}
          className="axis-label"
          transform="rotate(-90)"
        >
          {yAxisLabel}
        </text>
        <Marks
          xScale={xScale}
          yScale={yScale}
          data={data}
          selectColor={selectColor}
          xValue={xValue}
          yValue={yValue}
          tooltipFormat={xAxisTickFormat}
          circleRadius={7}
        />
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
